/********************************************************************************
    scsplittertest - extracts Video0 from program streams (to cout)
                     information about start-code positions go to cerr
 ********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include <libvideocoding/fileio/inputstream_istr.hh>
#include <libvideocoding/mpeg2/syntax/pessource.hh>
#include <libvideocoding/mpeg2/syntax/demux.hh>
#include <libvideocoding/mpeg2/syntax/scsplitter.hh>

#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include <iomanip>
#include <fstream>
using namespace std;
using namespace videocoding;

void Write(uint8* data,int len)
{
  cout.write((char*)data,len);
}

int main()
{
  try
    {
      InputStream_IStream istr;
      istr.SetIStream(cin);

      MP2_UniversalStream_Decoder progdec;
      progdec.SetInputStream(istr);
      progdec.ForceStreamType(MPEG_Program_Stream);

      MP2_PES_Demux demux;
      demux.SetPESSource(progdec);
      demux.EnableChannel(MP2_StartCode_Video0);

      MP2_PES_DemuxExtractor videoextr;
      videoextr.SetSource(demux);
      videoextr.SetChannel(MP2_StartCode_Video0);

      MP2_StartCodeSplitter splitter;
      splitter.SetSource(videoextr);
      //splitter.SetBreakpointAllSC();
      splitter.SetBreakpoint(MP2_StartCode_SequenceHeader);
      splitter.SetBreakpoint(MP2_StartCode_PictureStart,false);

      MP2_PES_JunkFilter junkfilter;
      junkfilter.SetSource(splitter);

      cerr << hex;

      for (;;)
	{
	  MP2_SystemPacket* pck = junkfilter.GetNextPacket();
	  if (pck==NULL)
	    break;

	  MP2_PES_Packet* pes = dynamic_cast<MP2_PES_Packet*>(pck);
	  Assert(pes);

	  cerr << setfill(' ');
	  cerr << setw(6) << pes->input_filepos << "+" << setw(4) << pes->data_start_offset << "=";
	  cerr << setw(6) << pes->input_filepos + pes->data_start_offset << ": ";

	  cerr << setfill('0');
	  for (int i=0;i<10;i++)
	    cerr << " " << setw(2) << ((int)pes->data.AskData()[i]);
	  cerr << endl;

	  Write(pes->data.AskData(), pes->data.AskLength());

	  delete pes;
	}
    }
  catch (const Excpt_Base& e)
    {
      MessageDisplay::Show(e);
    }

  return 0;
}
