/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/datatypes/dct.hh"

#include <string.h>
#include <iostream>
#include <iomanip>
using namespace std;

#include "config.h"

namespace videocoding {
  using namespace std;

  void DCTBlock_8x8::Clear()
  {
    memset(&coeff[0][0],0,64*sizeof(short));
    may_contain_data=false;
  }


  bool DCTBlock_8x8::IsAllZero() const
  {
#if 1
    // approx. 45% faster on my P-III than version below
    uint32* p = (uint32*)&coeff;

    uint32 val=0;

    for (uint16 i=0;i<64*sizeof(coeff[0][0])/4/4;i++)
      {
	val |= *p++;
	val |= *p++;
	val |= *p++;
	val |= *p++;
	if (val) return false;
      }

    return true;
#else
    for (int v=0;v<8;v++)
      for (int u=0;u<8;u++)
	if (coeff[v][u] != 0) { return false; }

    return true;
#endif
  }


  bool DCTBlock_8x8::operator==(const DCTBlock_8x8& blk) const
  {
    for (int v=0;v<8;v++)
      for (int u=0;u<8;u++)
	if (coeff[v][u] != blk.coeff[v][u])
	  return false;

    return true;
  }

  bool DCTBlock_8x8::operator!=(const DCTBlock_8x8& blk) const
  {
    return !(*this == blk);
  }

  bool DCTBlock_8x8::IsInRange(int low,int high) const
  {
    for (int i=0;i<64;i++)
      if (coeff[0][i] < low || coeff[0][i] > high)
	return false;
    return true;
  }

  bool DCTBlock_8x8::IsACInRange(int low,int high) const
  {
    for (int i=1;i<64;i++)
      if (coeff[0][i] < low || coeff[0][i] > high)
	return false;
    return true;
  }


  bool QMatrix_8x8::operator==(const QMatrix_8x8& q) const
  {
    for (int v=0;v<8;v++)
      for (int u=0;u<8;u++)
	if (mat[v][u] != q.mat[v][u])
	  return false;

    return true;
  }

  bool QMatrix_8x8::operator!=(const QMatrix_8x8& q) const
  {
    for (int v=0;v<8;v++)
      for (int u=0;u<8;u++)
	if (mat[v][u] != q.mat[v][u])
	  return true;

    return false;
  }


  ostream& operator<<(ostream& ostr,const DCTBlock_8x8& b)
  {
    ostr << setfill(' ');
    for (int v=0;v<8;v++,ostr << endl)
      for (int u=0;u<8;u++)
	ostr << setw(3) << ((int)b.coeff[v][u]) << ' ';

    return ostr;
  }

  ostream& operator<<(ostream& ostr,const QMatrix_8x8& q)
  {
    ostr << setfill(' ');
    for (int v=0;v<8;v++,ostr << endl)
      for (int u=0;u<8;u++)
	ostr << setw(2) << ((int)q.mat[v][u]) << ' ';

    return ostr;
  }
}
